import 'package:analyzer/dart/element/element.dart' as ele;
import 'package:source_gen/source_gen.dart';
import 'package:build/build.dart';
import 'package:dart_style/dart_style.dart';
import 'dart:io';
import 'formatter.dart';

class FormatterGenerator extends GeneratorForAnnotation<GFormatter> {
  final separator = Platform.pathSeparator;
  List<String> files = [];
  List result = [];
  List ignores = [
    '.entity.dart',
    '.route.dart',
    '.assets.dart',
    '.api.dart',
    '.iconfont.dart',
    '.interface.dart',
    '.formatter.dart',
    '.color.dart',
    '.size.dart',
  ];
  int totalCodeLines = 0;

  void queryFiles(String root, List<String> list, List<String> ignoresFiles,
      List<String> ignoresDirs) {
    try {
      var directory = Directory(root);
      List<FileSystemEntity> files = directory.listSync();
      files.forEach((f) {
        var bool = FileSystemEntity.isFileSync(f.path);
        var name = f.path.split("${separator}lib$separator").last;
        // 判断忽略文件夹列表
        var ignoresDir = ignoresDirs.indexOf(name) >= 0;
        if (!ignoresDir) {
          if (!bool) {
            queryFiles(f.path, list, ignoresFiles, ignoresDirs);
          } else {
            ///之后可以通过文件hash 来判断文件是否被变化，格式化代码效率
            ///忽略自动生成代码
            var ignoreItem;
            try {
              ignoreItem =
                  ignores.firstWhere((element) => f.path.endsWith(element));
            } catch (e) {}
            // 判断忽略文件列表
            var ignoresFile = ignoresFiles.indexOf(name) >= 0;
            if (!ignoresFile) {
              if (ignoreItem == null && f.path.endsWith('.dart')) {
                list.add(f.path);
              }
            }
          }
        }
      });
    } catch (e) {
      stdout.writeln("目录不存在!$root");
    }
  }

  void queryFileLines(String root, bool removeEmptyLines) {
    try {
      var directory = Directory(root);
      List<FileSystemEntity> files = directory.listSync();
      files.forEach((f) {
        var bool = FileSystemEntity.isFileSync(f.path);
        if (!bool) {
          queryFileLines(f.path, removeEmptyLines);
        } else {
          // 只统计dart文件
          if (f.path.endsWith('.dart')) {
            if (removeEmptyLines) {
              File(f.path).readAsLinesSync().forEach((element) {
                if (element.isNotEmpty && element.length > 0) {
                  totalCodeLines += 1;
                }
              });
            } else {
              totalCodeLines += File(f.path).readAsLinesSync().length;
            }
          }
        }
      });
    } catch (e) {
      stdout.writeln("目录不存在!$root");
    }
  }

  String? formatCode(codeStr) {
    var formatter = DartFormatter();
    String? codeF;
    try {
      codeF = formatter.format(codeStr);
    } on FormatterException catch (ex) {
      stdout.writeln(ex);
    }
    return codeF;
  }

  formatFile(path) async {
    int lines;

    ///使用同一个file操作
    File file = File(path);
    try {
      ///先格式化代码
      String codeStr = await file.readAsString();
      String? codeF = formatCode(codeStr);
      if (codeF == null) {
        return null;
      }

      ///写入文件
      file = await file.writeAsString(codeF);

      ///再统计代码行数
      List<String> linesCode = await file.readAsLines();
      lines = linesCode.length;
      result.add({'path': path, 'lines': lines});
    } catch (e) {
      stdout.writeln('读取文件失败! $path $e');
    }
  }

  void printResult(int maxLine) {
    int lengthW = 80;
    StringBuffer sb = StringBuffer();
    for (int i = 0; i < lengthW; i++) {
      if (i == 0) sb.write('╔');
      sb.write('═');
    }
    sb.write('\n║ 需优化代码行数的文件如下: [line >= $maxLine]');
    int index = 0;
    result.forEach((item) {
      var lines = item['lines'];
      if (lines >= maxLine) {
        index++;
        sb.write('\n║ [$index]\t[line:$lines]\t${item['path']}\t');
      }
    });
    for (int i = 0; i < lengthW; i++) {
      if (i == 0) sb.write('\n╚');
      sb.write('═');
    }
    if (index > 0) {
      stdout.writeln(sb.toString());
    }
  }

  @override
  generateForAnnotatedElement(ele.Element element, ConstantReader annotation,
      BuildStep buildStep) async {
    int maxLine = 0;
    var enable = false;
    List<String> ignoresFiles = [];
    List<String> ignoresDirs = [];
    var showTotalCodeLines = false;
    var removeEmptyLines = false;
    try {
      enable = annotation.read('enable').boolValue;
    } catch (e) {}
    if (!enable) return null;
    try {
      maxLine = annotation.read('maxLine').intValue;
    } catch (e) {}
    try {
      ignoresFiles = annotation
          .read('ignoresFiles')
          .listValue
          .map((e) => e.toStringValue()!)
          .toList();
    } catch (e) {}
    try {
      ignoresDirs = annotation
          .read('ignoresDirs')
          .listValue
          .map((e) => e.toStringValue()!)
          .toList();
    } catch (e) {}
    try {
      showTotalCodeLines = annotation.read('showTotalCodeLines').boolValue;
    } catch (e) {}
    try {
      removeEmptyLines = annotation.read('removeEmptyLines').boolValue;
    } catch (e) {}
    String root = '${Directory.current.path}${separator}lib';
    queryFiles(root, files, ignoresFiles, ignoresDirs);
    await Future.wait(
        files.map((path) async => await formatFile(path)).toList());
    if (result.length > 0) {
      printResult(maxLine);
    }
    if (showTotalCodeLines) {
      queryFileLines(root, removeEmptyLines);
      if (removeEmptyLines) {
        stdout.writeln("lib目录下除去空行所有源码行数:$totalCodeLines");
      } else {
        stdout.writeln("lib目录下包含空行所有源码行数:$totalCodeLines");
      }
    }
    return null;
  }
}

Builder formatterBuilder(BuilderOptions options) =>
    LibraryBuilder(FormatterGenerator(), generatedExtension: ".formatter.dart");
